import sys
#from io import BytesIO
#import gzip

from inforion.ionapi.controller import *
from inforion.ionapi.model import * 

from datetime import datetime, timedelta
import time

import inforion.ionapi.model.inforlogin as inforlogin
#import inforion.ionapi.basic as inforlogin

class TimeoutHTTPAdapter(HTTPAdapter):
    def __init__(self, *args, **kwargs):
        self.timeout = DEFAULT_TIMEOUT
        if "timeout" in kwargs:
            self.timeout = kwargs["timeout"]
            del kwargs["timeout"]
        super().__init__(*args, **kwargs)

    def send(self, request, **kwargs):
        timeout = kwargs.get("timeout")
        if timeout is None:
            kwargs["timeout"] = self.timeout
        return super().send(request, **kwargs)

def requests_retry_session(
    retries=3,
    backoff_factor=0.3,
    status_forcelist=(500, 502, 504),
    session=None,
    ):
    session = session or requests.Session()
    retry = Retry(
        total=retries,
        read=retries,
        connect=retries,
        backoff_factor=backoff_factor,
        status_forcelist=status_forcelist,
    )
    adapter = HTTPAdapter(max_retries=retry)
    session.mount('http://', adapter) 
    session.mount('https://', adapter)
    return session


    

def sendresults(url,headers, data,timeout=65,stream=False):
    
    retry_strategy = Retry(
        total=5,
        backoff_factor=1,
        status_forcelist=[429, 500, 502, 503, 504],
        method_whitelist=["HEAD", "POST","GET", "OPTIONS"]
    )
    adapter = HTTPAdapter(max_retries=retry_strategy,pool_connections=100,pool_maxsize=100)
    http = requests.Session()
    http.mount("https://", adapter)
    http.mount("http://", adapter)


    if datetime.now() > inforlogin._GLOBAL_session_expire:
        
        headers = inforlogin.reconnect()
        print (" Reconnect and Next Reconnect will be " + str(inforlogin._GLOBAL_session_expire))
        
   

    try:
        for z in range(0,5):           
            #print (inforlogin.header())
            response = http.request("POST", url, headers=inforlogin.header(), data=json.dumps(data),timeout=timeout,stream=stream)

            if response.status_code == 200:
                try:
                    r =  response.json()   
                    break     
    
                except ValueError:
                    print (r)
                    r = "JSON Error"
            else:
                 
                if z < 5:
                    print (" Error try to get new session "+ str(z) + "/5")
                    headers = inforlogin.reconnect()
                    time.sleep(10)     
                elif z == 5:
                    sys.exit(0)    

    except requests.exceptions.TooManyRedirects:
        print ("Too many redirects")
        r = "Error - Too many redirects"
        raise SystemExit(e)
    except requests.exceptions.RequestException as e:
        # catastrophic error. bail.
        print ("OOps: Something Else",e)
        raise SystemExit(e)
        r = "Error"
    
    return r

def saveresults(r,df,program,index,chunk,MaxChunk=150,elements=1):

    message = ''
    max_elements = elements
    try:
        if chunk == 0:
            newindex = index - MaxChunk
        else:
            newindex = index - MaxChunk + chunk
        if newindex < 0: 
            newindex = 0
        cmethod = None
        if len(r)>0:
            if 'results' in r.keys():
                
                if len(r['results'])>0:

                    for key in r['results']:

                        methode = key['transaction']
                        
                        
                        if 'errorMessage' in key:
                            error = key['errorMessage']
                            error = error.rstrip("\r\n")
                            error = ' '.join(error.split())
                            message += methode+':'+error+'|'

                        else:
                            message += methode+':OK|'
                        
                        df.loc[newindex, 'MESSAGE'] = message
                        
                        if elements == 1:
                            newindex = newindex + 1
                            elements = max_elements
                            message = ''
                        else:
                            elements = elements - 1

              
                else:
                    
                    df.loc[df.index.to_series().between(newindex,index), 'MESSAGE'] = "Results are empty"
            else:
                
                df.loc[df.index.to_series().between(newindex,index), 'MESSAGE'] = "Results are missing"
        else:
            for newindex in range(index):
                #print('Write JSON Error:', str(newindex))
                df.loc[newindex, 'MESSAGE'] = ' JSON Error'
    except:
        print (r)
        df.loc[df.index.to_series().between(newindex,index), 'MESSAGE'] = 'Unclear Error'


    chunk = MaxChunk
    data = {'program': program,
    'cono':    409 }
    
    return df, data,chunk






import sys

#from inforion.ionapi.ionbasic import ionbasic

#from ionbasic import load_config

# Codee Junaid 

import inforion.ionapi.model.inforlogin as inforlogin


from inforion.transformation.transform import parallelize_tranformation
from inforion.ionapi.controller import *
from inforion.ionapi.model import * 
from inforion.helper.urlsplit import spliturl


import validators
import os.path



def main_load(url=None,ionfile=None,program=None,method=None,dataframe=None,outputfile=None,start=None,end=None):
     
    
    if validators.url(url) != True:
        return ("Error: URL is not valid")
    
    if os.path.exists(ionfile) == False:
        return ("Error: File does not exist")
    else:
        config = inforlogin.load_config(ionfile)
        

    
    result = spliturl(url)

    if "Call" in result:
        if len(result["Call"]) > 0:
            if result["Call"] == "execute":
                config = inforlogin.load_config(ionfile)
                token =  inforlogin.login()
                
                headers = inforlogin.header()
                if "Bearer"  not in headers['Authorization']:
                    return "Error: InforION Login is not working"
                if start is None or end is None:
                    return execute(url,headers,program,method,dataframe,outputfile)  
                    
                else:
                    return execute(url,headers,program,method,dataframe,outputfile,start,end)  
                    
            if result["Call"] == "executeSnd":

                config = inforlogin.load_config(ionfile)
                token = inforlogin.login()
                
                headers = inforlogin.header(token)
                if "Bearer"  not in headers['Authorization']:
                    return "InforION Login is not working"
                return executeSnd(url,headers,program,method,dataframe,outputfile,start,end)  
            if result["Call"] == "executeAsyncSnd":
        
                config = inforlogin.load_config(ionfile)
                token= inforlogin.login()
                
                headers = inforlogin.header(token)
                if "Bearer"  not in headers['Authorization']:
                    return "InforION Login is not working"
                return executeAsyncSnd(url,headers,program,method,dataframe,outputfile,start,end)

    if method == "checklogin":
        token= inforlogin.login()
        headers = inforlogin.header()
        return (headers['Authorization'])

def main_transformation(mappingfile=None,mainsheet=None,stagingdata=None,outputfile=None):
        
    if mappingfile is None:
        return ("Error: Mapping file path missing")
    
    if os.path.exists(mappingfile) == False:
        return ("Error: Mapping file does not exist")
    
    if mainsheet is None:
        return("Error: Main sheet name is empty")
    
    if stagingdata.empty:
        return("Error: Data frame is empty")
    
    return (parallelize_tranformation(mappingfile,mainsheet,stagingdata,outputfile))






# Transformation of Staging Data into M3 format via mapping file 
import pandas as pd
import json
import numpy as np
import decimal
import datetime
from multiprocessing import  Pool
from functools import partial

def parallelize_tranformation(mappingfile, mainsheet,stagingdata,outputfile=None,n_cores=4):
    # Read the file from given location
    xls = pd.ExcelFile(mappingfile)

    # to read all sheets to a map
    sheet_to_df_map = {}
    for sheet_name in xls.sheet_names:
        sheet_to_df_map[sheet_name] = xls.parse(sheet_name)

    main_cache = getMainSheetCache(sheet_to_df_map, mainsheet)
    tabs_cache = getTabsMappingCache(sheet_to_df_map, main_cache)

    df_split = np.array_split(stagingdata, n_cores)
    func = partial(transform_data, sheet_to_df_map, mainsheet, main_cache, tabs_cache)
    pool = Pool(n_cores)
    df = pd.concat(pool.map(func, df_split))
    pool.close()
    pool.join()
    
    if outputfile is not None:
        print ('Save to file: ' + outputfile)
        writer = pd.ExcelWriter(outputfile, engine='xlsxwriter')
        df.to_excel(writer, sheet_name='Log Output',index=False)
        writer.save()

    return df

def getMainSheetCache(sheet_to_df_map, mainsheet):
    mapping_cache = []

    for index, row in sheet_to_df_map[mainsheet].iterrows():
        if index >=9:
            row = row.replace(np.nan, '', regex=True)

            map = {}
            map['API_FIELD'] = row[15]

            if row[33] and not row[33] is np.nan:
                map['SOURCE'] = row[33]
            else:
                map['SOURCE'] = None

            map['FUNC_TYPE'] = row[36].strip().lower()

            map['FUNC_VAL'] = row[37]
            map['FUNC_ARG'] = row[38]

            mapping_cache.append(map)
    
    return mapping_cache

def getTabsMappingCache(sheet_to_df_map, mapping_cache):
    mapping_sheets_cache = {}
    
    for map in mapping_cache:
        if map['FUNC_TYPE'] == 'tbl':
            tab_key = map['FUNC_VAL'].strip()
            if tab_key not in mapping_sheets_cache:
                tab = {}
                for i, val in sheet_to_df_map[tab_key].iterrows():
                    if i >= 7:
                        if str(val[0]) == 'nan': val[0] = ''
                        tab[str(val[0])] = str(val[1])
                mapping_sheets_cache[tab_key] = tab
    
    return mapping_sheets_cache


def transform_data(sheet_to_df_map, mainsheet, sheet_cache, tabs_cache, stagingdata):
    rows_list = []

    for tb_index,tb_row in stagingdata.iterrows():
        row_dict = {}

        for map in sheet_cache:

            if map['SOURCE']:
                row_dict[map['API_FIELD']] = str(tb_row[map['SOURCE']])
            else:
                if map['FUNC_TYPE'] == 'tbl':
                    if map['FUNC_ARG'] and not map['FUNC_ARG'] is np.nan:
                        tab = tabs_cache[map['FUNC_VAL']]
                        if tb_row[map['FUNC_ARG']] in tab:
                            row_dict[map['API_FIELD']] = str(tab[tb_row[map['FUNC_ARG']]])
                        elif '*' in tab:
                            row_dict[map['API_FIELD']] = str(tab['*'])
                        else:
                            row_dict[map['API_FIELD']] = str(tb_row[map['FUNC_ARG']])
                elif map['FUNC_TYPE'] == 'func':
                    if map['FUNC_VAL'].strip().lower() == "div":
                        data_values = map['FUNC_ARG'].split('|')
                        with decimal.localcontext() as ctx:
                            if data_values[2] != "":
                                ctx.prec = int(data_values[2])
                            division = decimal.Decimal(tb_row[data_values[0]]) / decimal.Decimal(data_values[1])
                    row_dict[map['API_FIELD']] = division
                elif map['FUNC_TYPE'] == 'const':
                    if isinstance(map['FUNC_VAL'], datetime.datetime):
                        val = map['FUNC_VAL'].strftime("%Y%m%d")
                        row_dict[map['API_FIELD']] = str(val)
                    else:
                        row_dict[map['API_FIELD']] = str(map['FUNC_VAL'])


        rows_list.append(row_dict)

    df = pd.DataFrame(rows_list).replace('nan', '', regex=True)
    
    return df


import requests
from inforion.ionapi.model import inforlogin


# Retrieving data objects
# /v1/payloads/streambyfilter
# /v1/payloads/streambyid
# /v1/payloads/markcorrupt

# /v1/payloads/list
def get_v1_payloads_list(filter=None, sort=None, page=None, records=None):
    url = inforlogin.base_url() + "/IONSERVICES/datalakeapi/v1/payloads/list"
    headers = inforlogin.header()
    payload = {}

    if filter is not None:
        payload["filter"] = filter

    if sort is not None:
        payload["sort"] = sort

    if page is not None:
        payload["page"] = page

    if records is not None:
        payload["records"] = records

    res = requests.get(url, headers=headers, params=payload)
    return res


# Querying data objects
# /v1/compass/ping
# /v1/compass/jobs
# /v1/compass/jobs/{queryId}/status
# /v1/compass/jobs/{queryId}/result


# Purging data objects
# /v1/purge/ids
# /v1/purge/filter

# Archiving data objects
# /v1/archive/strategy
# /v1/archive/strategy
# /v1/archive/logs

# Restoring data objects
# /v1/restore/list
# /v1/restore/payloads
# /v1/restore/logs




import sys 

import json

from requests_oauthlib import OAuth2Session
from requests.auth import HTTPBasicAuth
from oauthlib.oauth2 import BackendApplicationClient

import requests

from datetime import datetime, timedelta


this = sys.modules[__name__]


this._GLOBAL_access_token = None
this._GLOBAL_expires_in = None
this._GLOBAL_refresh_token = None
this._GLOBAL_token_type = None
this._GLOBAL_start_session = None
this._GLOBAL_session_expire = None
this._GLOBAL_saak = None
this._GLOBAL_sask = None
this._GLOBAL_client_id = None
this._GLOBAL_client_secret = None
this._GLOBAL_ti = None
this._GLOBAL_cn = None

# Name of the URL
this._GLOBAL_url = None
# URL SSO
this._GLOBAL_sso_url = None

# directory for token 
this._GLOBAL_token_outh2 = None


def configfile(filename):
    with open(filename) as file:
        config_json = json.load(file)
                
    if all (k in config_json for k in ('url','ionfile','program','method','inputfile')):
        typ = "L"
        url = config_json['url']
        ionfile = config_json['ionfile']
        program = config_json['program']
        method = config_json['method']
        inputfile = config_json['inputfile']
        outputfile = config_json['outputfile']
    
        
    else:
        print ("JSON File wrong config")
        sys.exit(0)
    
    if "start" in config_json:
            start = config_json["start"]
    else:
        start = None
    if "end" in config_json:
        end = config_json["end"]
    else:
        end = None


def update(ti, cn, access_token, expires_in,refresh_token,token_type,start_session,session_expire,saak,sask,client_id,client_secret):
    this._GLOBAL_ti = ti
    this._GLOBAL_cn = cn
    this._GLOBAL_access_token = access_token
    this._GLOBAL_expires_in = expires_in
    this._GLOBAL_refresh_token = refresh_token
    this._GLOBAL_token_type = token_type
    this._GLOBAL_start_session = start_session
    this._GLOBAL_session_expire = session_expire
    this._GLOBAL_saak = saak
    this._GLOBAL_sask = sask
    this._GLOBAL_client_id = client_id
    this._GLOBAL_client_secret = client_secret

def load_config(IONFile):
    with open(IONFile) as json_file:
        data = json.load(json_file)
        if "ti" not in data:
            print ("Error in ION file - ti")            
            sys.exit(0)
        else:
            this._GLOBAL_ti = data["ti"]
        if "cn" not in data:
            print ("Error in ION file - cn")            
            sys.exit(0)
        else:
            this._GLOBAL_cn = data["cn"]

        this._GLOBAL_client_id = data["ci"] 
        this._GLOBAL_client_secret = data["cs"] 
        this._GLOBAL_saak = data["saak"]
        this._GLOBAL_sask = data["sask"]
        this._GLOBAL_url = data["iu"]
        this._GLOBAL_sso_url = data["pu"]
        this._GLOBAL_token_outh2 = data["ot"]

    return data 


def login():
    
    

    start_session = datetime.now()

    

    url = this._GLOBAL_sso_url + this._GLOBAL_token_outh2
    

    data = {
                'grant_type' : 'password',
                'username' : this._GLOBAL_saak,
                'password' : this._GLOBAL_sask ,
                'client_id' : this._GLOBAL_client_id,
                'client_secret' : this._GLOBAL_client_secret,
                'scope' : '',
                'redirect_uri' : 'https://localhost/'
    }

    #print (data)
    r = requests.post(url, data=data)
    
    r = r.json()

    access_token = r['access_token']
    expires_in = r['expires_in']
    refresh_token = r['refresh_token']
    token_type = r['token_type']
    saak = this._GLOBAL_saak
    sask = this._GLOBAL_sask
    client_id = this._GLOBAL_client_id
    ti = this._GLOBAL_ti
    cn = this._GLOBAL_cn
    client_secret = this._GLOBAL_client_secret
  
    session_expire = addSecs(start_session, expires_in)
    #session_expire = addSecs(start_session, 60)  

    
    update(ti, cn, access_token, expires_in, refresh_token, token_type,start_session,session_expire,saak,sask,client_id,client_secret)

    return r


def addSecs(tm, secs):
    
    if type(secs) != int:
        secs = int(secs)
    expires_date = tm + timedelta(seconds=secs)
    
    return expires_date



def reconnect():
    start_session = datetime.now()


    url = this._GLOBAL_sso_url + this._GLOBAL_token_outh2

   
    
    refresh_token = this._GLOBAL_refresh_token
    saak = this._GLOBAL_saak
    sask = this._GLOBAL_sask
    client_id = this._GLOBAL_client_id
    client_secret = this._GLOBAL_client_secret
    ti = this._GLOBAL_ti
    expires_in = this._GLOBAL_expires_in

    data = {
                'grant_type' : 'refresh_token',
                'refresh_token' : refresh_token,
                'username' : saak,
                'password' : sask,
                'client_id' : client_id,
                'client_secret' : client_secret,
                'scope' : '',
                'redirect_uri' : 'https://localhost/'
    }

    #print (data)
    r = requests.post(url, data=data)
    
    r = r.json()
    #print (r)

    this._GLOBAL_session_expire = addSecs(start_session, r['expires_in'])
    if 'access_token' not in r:
        print ('Error Reconnect Json')
        sys.exit(0)
    else:
        this._GLOBAL_access_token = r['access_token']

     

    #update(ti,access_token, expires_in, refresh_token, token_type,start_session,session_expire,saak,sask,client_id,client_secret)

    headers = header()

    #print (" New session ")
    #print (headers['access_token'])

    return headers




def header():
    '''
        headers = {
        'Content-Type': 'application/json',
        'X-TenantId': 'FELLOWCONSULTING_DEV',
        'X-ClientId': 'FELLOWCONSULTING_DEV~t1YYSUtR23J-h92WoUUQVPEOJQJZf7l5qjzZkbCrq8I',
        'Authorization': 'Bearer eyJhbGciOiJSUzI1NiIsImtpZCI6IkluZm9yQWNjZXNzVG9rZW5TaWduaW5nQ2VydGlmaWNhdGUtMTU3NjM2MzI3NyJ9.eyJzY29wZSI6Im51bGwiLCJjbGllbnRfaWQiOiJGRUxMT1dDT05TVUxUSU5HX0RFVn50MVlZU1V0UjIzSi1oOTJXb1VVUVZQRU9KUUpaZjdsNXFqelprYkNycThJIiwianRpIjoiM2NiYlVyVEx4ZERvbEFOUVM4aGs3MVlSRUpEd3UyWjBzZ3dxIiwiU2VydmljZUFjY291bnQiOiJGRUxMT1dDT05TVUxUSU5HX0RFViN4dGQ3QVZjeGFSVXlnQkRxVXBLdkJab0dXd08wZ2NHZ0U5UTNDaHdnSlREZ1VXUWNHZ2ptZjBGd1poZW9wSHI1ZmZrUVBmejFBOHZLYjExTEh0QkstdyIsIklkZW50aXR5MiI6IjQ2YjYwMmE0LWIxNTgtNGQ2My05YjJlLTZiMjJkYmEwNjU0YyIsIlRlbmFudCI6IkZFTExPV0NPTlNVTFRJTkdfREVWIiwiRW5mb3JjZVNjb3Blc0ZvckNsaWVudCI6IjAiLCJleHAiOjE1ODIyMTM3MjB9.FMBtAWQVh-S81kEDRjZnN3rujAGyX0UIj5SKLvEhlTLOu0jT92JRB5VuHKRHtKg-ODcDgSMd2i1YMcALFcQvxiTRyvo3oW3m5GAaELv_TUWcr7r-Qd952WIQUAtfTY66CUclWYIkHa3HuEF2t9m7Doglw88RakcqHAK8-SnONJTF9UreVD6ZO3sVFu7UDB5DKOr6iwfZPFJtTJGaiTpLydXqTto6vGaZ3csC0zx6IfPqSKQg7yfz9u1I_mJbdG6PnmrksBVWkCF6lG30ibdM2jCjhELuzWlXHjTD47n4K84O9OvuylYG2wuT8DDHlL255oOLzFkySdqtMAbdXKveyQ',
        'User-Agent': 'PostmanRuntime/7.22.0',
        'Accept': '*/*',
        'Cache-Control': 'no-cache',
        'Postman-Token': '5ebf61ab-bb78-4b14-9161-bf5ee714c210',
        'Host': 'mingle-ionapi.eu1.inforcloudsuite.com',
        'Accept-Encoding': 'gzip, deflate, br',
        'Content-Length': '359',
        'Cookie': 'useractivity_cookie_mingle=1582207527',
        'Connection': 'keep-alive'
        }
    '''

    headers = {
        'Content-Type':         'application/json',
        'X-TenantId':           this._GLOBAL_ti,
        'Connection':           'keep-alive',
        'Host':                 'mingle-ionapi.eu1.inforcloudsuite.com',
        'Accept-Encoding':      'gzip, deflate, br',
        'Cache-Control':        'no-cache',
        'User-Agent':           'FellowConsultingAGRuntime/7.22.0',
        'X-ClientId':           this._GLOBAL_client_id,
        'Accept':               '*/*',
        'Authorization':        'Bearer ' + this._GLOBAL_access_token,

    }
    return headers


def base_url():
    return this._GLOBAL_url + '/' + this._GLOBAL_ti




import pandas as pd
import numpy as np
import requests
import inforion
import json

import time
import progressbar

#import grequests

from pandas import compat

import xlsxwriter

from requests_oauthlib import OAuth2Session
from requests.auth import HTTPBasicAuth
from oauthlib.oauth2 import BackendApplicationClient



from requests.adapters import HTTPAdapter
from requests.packages.urllib3.util.retry import Retry

import inforion.ionapi.controller as controller
import inforion.helper.filehandling as filehandling
#import sendresults, saveresults
#from inforion.ionapi.model import 

DEFAULT_TIMEOUT = 50 # seconds
MaxChunk = 100



def execute(url,headers,program,methode,dataframe,outputfile=None,start=0,end=None):
    
    df = dataframe
    
    df = df.replace(np.nan, '', regex=True)
    df = df.astype(str)

    data = {'program': program,
            'cono':    409 }
        

    
    mylist = []
    data1 = {}
    data2 = {}
    a = []
   
    
    chunk = MaxChunk
    if end is not None:
        #total_rows = end - start
        counter = 0
        df = df[start:end].copy(deep=False)
        df = df.reset_index(drop=True)
        #print (df.head(10))
        
    #else:
    total_rows = df.shape[0]
    total_rows = int(total_rows)
    
    methode = methode.split(",")
    methode_count = len(methode)

    print ("Number of rows " + str(total_rows))

    
    
    with progressbar.ProgressBar(max_value=total_rows) as bar:
        for index,row in df.iterrows():
            
            bar.update(index)
                
            
            row = row.to_json()
            row = json.loads(row)

            

            
            
            for i in methode:
                data1['transaction'] = i
                data1['record'] = row
                a.append(data1.copy())
                

            

            if chunk == 0: 
                data['transactions'] = a

                r = controller.sendresults(url,headers,data)
                df,data,chunk = controller.saveresults(r,df,program,index,chunk,MaxChunk,methode_count)
                data1 = {}
                a = []
                 
            else:
                chunk = chunk - 1
        
               
        
        data['transactions'] = a
        
        r = controller.sendresults(url,headers,data)
        index = index + 1 
        df,data,chunk = controller.saveresults(r,df,methode,index,chunk,MaxChunk,methode_count)


    #df = df.replace(np.nan, '', regex=True)
    #df = df.astype(str)

    if outputfile is not None:
        print ('Save to file: ' + outputfile)
        filehandling.savetodisk(outputfile,df)
    
    return df

    
def executeSnd(url,headers,program,methode,dataframe,outputfile=None,start=0,end=None):
    
    df = dataframe


    data = {'program': program,
            'cono':    409 }
        


    methode = methode.split(",")
    methode_count = len(methode)
    
    mylist = []
    data1 = {}
    data2 = {}

  
    
    chunk = MaxChunk
    if end is not None:
        #total_rows = end - start
        counter = 0
        df = df[start:end].copy(deep=False)
        df = df.reset_index(drop=True)
        
        
    #else:
    total_rows = df.shape[0]
    total_rows = int(total_rows)
    
    

    print ("Number of rows " + str(total_rows))

    
    a = []
    with progressbar.ProgressBar(max_value=total_rows) as bar:
        for index,row in df.iterrows():
            
            bar.update(index)
                
            
            row = row.to_json()
            row = json.loads(row)

            for i in methode:
                data1['transaction'] = i
                data1['record'] = row
                a.append(data1.copy())
            
           

                
        
        data['transactions'] = a
    
        index = index + 1 
        
    
    print (data)

    r = controller.sendresults(url,headers,data)


    df,data,chunk = controller.saveresults(r,df,methode,index,chunk)

    df = df.replace(np.nan, '', regex=True)
    df = df.astype(str)

    if outputfile is not None:
        print ('Save to file: ' + outputfile)
        filehandling.savetodisk(outputfile,df)
    
    return df

def executeAsyncSnd(url,headers,program,methode,dataframe,outputfile=None,start=0,end=None):

    print ("Still in Beta")

    df = dataframe
        
    data = {'program': program,
            'cono':    409 }
        
    methode = methode.split(",")
    methode_count = len(methode)
    
    mylist = []
    data1 = {}
    data2 = {}

  
    
    chunk = MaxChunk
    if end is not None:
        #total_rows = end - start
        counter = 0
        df = df[start:end].copy(deep=False)
        df = df.reset_index(drop=True)
        #print (df.head(10))
        
    #else:
    total_rows = df.shape[0]
    total_rows = int(total_rows)
    
    

    print ("Number of rows " + str(total_rows))

    
    a = []
    with progressbar.ProgressBar(max_value=total_rows) as bar:
        for index,row in df.iterrows():
            
            bar.update(index)
                
            
            row = row.to_json()
            row = json.loads(row)

            for i in methode:
                data1['transaction'] = i
                data1['record'] = row
                a.append(data1.copy())

            

            

            
        
               
        
        data['transactions'] = a


        print (data)

        r = controller.sendresults(url,headers,data,stream=True)
        index = index + 1 
        #df,data,chunk = saveresults(r,df,methode,index,chunk)

    print (r)

    '''

    df = df.replace(np.nan, '', regex=True)
    df = df.astype(str)
    
    if outputfile is not None:
        print ('Save to file: ' + outputfile)
        filehandling.savetodisk(outputfile,df)
    
    return df

    '''




import urllib.parse

import os 

#url = "https://mingle-ionapi.eu1.inforcloudsuite.com/BVB_DEV/M3/m3api-rest/v2/execute"


def spliturl(url):
    result = {}
    path = urllib.parse.urlparse(url).path
    
    t = os.path.normpath(path).split(os.path.sep)


    z=0
    for i in t:
        if len(i)>0:
            if z == 1:
                result["Company"] = i
            elif z == 2:
                result["App"] = i
            elif z == 3:
                result["Type"] = i
            elif z == 4:
                result["Version"] = i
            elif z == 5:
                result["Call"] = i
        z = z +1
        

    return result

